==> raspberryclient-1.0.0.dev4-py2.py3-none-any.whl: raspberryclient/python_client.py <==
__version__ = "1.0.0.dev4"

import os
import sys
import time
import logging
import traceback
import subprocess
from logging.handlers import RotatingFileHandler

try:
    import thread
except ImportError:
    import _thread as thread

import websocket

from .websocket_communication import WebsocketCommunication

LOGGER_NAME = "Python_port_logger"
FORMATTER = logging.Formatter('[%(asctime)s] {%(pathname)s:%(lineno)d} %(levelname)s - %(funcName)s -  %(message)s')

no_internet = "./no_internet.mp3"
lost_internet = "./lost_internet.mp3"
interspeech_sound = "./interspeech_sound.mp3"


class PythonClient:
    communication = None
    LED_PIN = 25

    def __init__(self, device_id):
        self.DEVICE_ID = device_id
        self.HOST = sys.argv[1] if len(sys.argv) > 1 else "wss://port.preview.promethist.ai/channel/"

        self.LOG_FILENAME = "./logs/" + self.DEVICE_ID + "_debug.txt"

        self.logger = self.init_logger(LOGGER_NAME)

        self.should_stop_audio = False
        self.session_started = False

        self.led = None
        self.button = None
        self.init_peripheral()

    def init_peripheral(self):
        # Raspberry PI specific
        try:
            from gpiozero import LED, Button

            self.led = LED(self.LED_PIN)
            self.button = Button(23)

        # MacOS
        except ImportError:
            class FakeLED:

                def on(self):
                    # print("Led On")
                    pass

                def off(self):
                    # print("Led Off")
                    pass

            class FakeButton:
                def __init__(self):
                    self.when_pressed = lambda x: print("was clicked")

                def wait_for_press(self):
                    time.sleep(5)

            self.led = FakeLED()
            self.button = FakeButton()

            from pynput import keyboard  # has to run as root because of permission

            def on_press(key):
                print('key {0} pressed'.format(key))
                if "{0}".format(key) == "Key.cmd":
                    self.button.when_pressed()

            # ...or, in a non-blocking fashion:
            listener = keyboard.Listener(
                on_press=on_press)
            listener.start()

    def get_console_handler(self):
        console_handler = logging.StreamHandler(sys.stdout)
        console_handler.setFormatter(FORMATTER)
        console_handler.setLevel(logging.DEBUG)
        return console_handler

    def get_file_handler(self):
        file_handler = RotatingFileHandler(self.LOG_FILENAME, maxBytes=10000000, backupCount=10)
        file_handler.setFormatter(FORMATTER)
        file_handler.setLevel(logging.DEBUG)
        return file_handler

    def init_logger(self, logger_name):
        os.makedirs("temp", exist_ok=True)
        os.makedirs("logs", exist_ok=True)

        logger = logging.getLogger(logger_name)
        logger.setLevel(logging.DEBUG)  # better to have too much log than not enough
        logger.addHandler(self.get_console_handler())
        logger.addHandler(self.get_file_handler())
        # with this pattern, it's rarely necessary to propagate the error up to parent
        logger.propagate = False
        return logger

    def button_pressed(self):
        """
        The user pressed the button. It can happen in two situation. First is when the
        :return:
        """
        if self.session_started:
            self.logger.debug("button_pressed(session_started == True)")
            self.end_session()
            self.communication.force_stop()
        else:
            self.logger.debug("button_pressed(session_started == False)")
            try:
                self.led.on()
                self.communication.force_push()
                while self.communication.is_session_started():
                    self.logger.debug("sessionId:{}".format(self.communication.session_id))
                    time.sleep(0.1)
                time.sleep(1)
                self.led.off()
            except Exception as e:
                self.logger.error(e)
                self.led.on()
                time.sleep(0.5)
                self.led.off()
                time.sleep(0.5)
                self.led.on()
                time.sleep(0.5)
                self.led.off()

    def play_audio(self, src):
        """
        Play audio with possibility to kill the process
        :param src:
        :return: Subprocess playing audio which can be killed
        """
        cmd = 'exec mpg321 ' + src
        return subprocess.Popen(cmd, stdout=subprocess.PIPE, shell=True)

    def play_bytes_with_interruption(self, audio_bytes):
        """
        Transforms the bytes to mp3 file and start playing it
        :param audio_bytes:
        :return: True if the whole audio was played. False if we press interrupt button during listening the audio
        """
        self.logger.info("play_bytes")

        src = "./temp/" + self.DEVICE_ID + "_" + str(time.time()) + ".mp3"
        with open(src, "wb") as f:
            # Write the response to the output file.
            f.write(audio_bytes)

        p = self.play_audio(src)

        while p.poll() is None:
            if self.should_stop_audio:
                self.logger.debug("should_stop_audio==True")
                p.kill()
                os.remove(src)
                self.should_stop_audio = False
                return False
            else:
                # self.logger.debug("sleeping")
                time.sleep(0.1)

        os.remove(src)
        return True

    def start_session(self):
        self.session_started = True
        self.should_stop_audio = False

    def end_session(self):
        self.session_started = False
        self.should_stop_audio = True

    def clean_up_after_error(self):
        self.session_started = False

    def download_offline_sound(self):
        # Download the file if it does not exist
        if not os.path.isfile(no_internet):
            import urllib.request
            urllib.request.urlretrieve("https://promethist.ai/repository/misc/no_internet.mp3", no_internet)
        if not os.path.isfile(lost_internet):
            import urllib.request
            urllib.request.urlretrieve("https://promethist.ai/repository/misc/lost_internet.mp3", lost_internet)
        if not os.path.isfile(interspeech_sound):
            import urllib.request
            urllib.request.urlretrieve("https://promethist.ai/repository/misc/interspeech_sound.mp3", interspeech_sound)

    def run(self):
        self.download_offline_sound()

        self.button.when_pressed = self.button_pressed

        self.communication = WebsocketCommunication(host=self.HOST,
                                                    logger_name=LOGGER_NAME,
                                                    device_id=self.DEVICE_ID,
                                                    on_alert_on=self.led.on,
                                                    on_alert_off=self.led.off,
                                                    on_session_start=self.start_session,
                                                    on_session_end=self.end_session,
                                                    play_bytes=self.play_bytes_with_interruption,
                                                    play_audio=self.play_audio)
        while True:
            if "DEBUG" in os.environ:
                websocket.enableTrace(True)
            try:
                thread.start_new_thread(self.communication.run, ())
                while True:
                    print(str(self.communication.websocket.run_forever(ping_interval=60, ping_timeout=20)))
                    time.sleep(5)
                    self.logger.debug("Connection with port is lost or unavailable!")
            except ConnectionRefusedError as e:
                self.play_audio(no_internet)
                time.sleep(20)
            except (SystemExit, Exception):
                self.end_session()
                self.logger.error(traceback.format_exc())
                self.communication.websocket.close()
                time.sleep(4)

==> raspberryclient-1.0.1-py2.py3-none-any.whl: raspberryclient/python_client.py <==
__version__ = "1.0.1"

import os
import sys
import time
import logging
import traceback
import subprocess
from logging.handlers import RotatingFileHandler

try:
    import thread
except ImportError:
    import _thread as thread

import websocket

from .websocket_communication import WebsocketCommunication

LOGGER_NAME = "Python_port_logger"
FORMATTER = logging.Formatter('[%(asctime)s] {%(pathname)s:%(lineno)d} %(levelname)s - %(funcName)s -  %(message)s')

no_internet = "./no_internet.mp3"
lost_internet = "./lost_internet.mp3"
interspeech_sound = "./interspeech_sound.mp3"


class PythonClient:
    communication = None
    LED_PIN = 25

    def __init__(self, device_id):
        self.DEVICE_ID = device_id
        self.HOST = sys.argv[1] if len(sys.argv) > 1 else "wss://port.preview.promethist.ai/channel/"

        self.LOG_FILENAME = "./logs/" + self.DEVICE_ID + "_debug.txt"

        self.logger = self.init_logger(LOGGER_NAME)

        self.should_stop_audio = False
        self.session_started = False

        self.led = None
        self.button = None
        self.init_peripheral()

    def init_peripheral(self):
        # Raspberry PI specific
        try:
            from gpiozero import LED, Button

            self.led = LED(self.LED_PIN)
            self.button = Button(23)

        # MacOS
        except ImportError:
            class FakeLED:

                def on(self):
                    # print("Led On")
                    pass

                def off(self):
                    # print("Led Off")
                    pass

            class FakeButton:
                def __init__(self):
                    self.when_pressed = lambda x: print("was clicked")

                def wait_for_press(self):
                    time.sleep(5)

            self.led = FakeLED()
            self.button = FakeButton()

            from pynput import keyboard  # has to run as root because of permission

            def on_press(key):
                print('key {0} pressed'.format(key))
                if "{0}".format(key) == "Key.cmd":
                    self.button.when_pressed()

            # ...or, in a non-blocking fashion:
            listener = keyboard.Listener(
                on_press=on_press)
            listener.start()

    def get_console_handler(self):
        console_handler = logging.StreamHandler(sys.stdout)
        console_handler.setFormatter(FORMATTER)
        console_handler.setLevel(logging.DEBUG)
        return console_handler

    def get_file_handler(self):
        file_handler = RotatingFileHandler(self.LOG_FILENAME, maxBytes=10000000, backupCount=10)
        file_handler.setFormatter(FORMATTER)
        file_handler.setLevel(logging.DEBUG)
        return file_handler

    def init_logger(self, logger_name):
        os.makedirs("temp", exist_ok=True)
        os.makedirs("logs", exist_ok=True)

        logger = logging.getLogger(logger_name)
        logger.setLevel(logging.DEBUG)  # better to have too much log than not enough
        logger.addHandler(self.get_console_handler())
        logger.addHandler(self.get_file_handler())
        # with this pattern, it's rarely necessary to propagate the error up to parent
        logger.propagate = False
        return logger

    def button_pressed(self):
        """
        The user pressed the button. It can happen in two situation. First is when the
        :return:
        """
        if self.session_started:
            self.logger.debug("button_pressed(session_started == True)")
            self.end_session()
            self.communication.force_stop()
        else:
            self.logger.debug("button_pressed(session_started == False)")
            try:
                self.led.on()
                self.communication.force_push()
                while self.communication.is_session_started():
                    self.logger.debug("sessionId:{}".format(self.communication.session_id))
                    time.sleep(0.1)
                time.sleep(1)
                self.led.off()
            except Exception as e:
                self.logger.error(e)
                self.led.on()
                time.sleep(0.5)
                self.led.off()
                time.sleep(0.5)
                self.led.on()
                time.sleep(0.5)
                self.led.off()

    def play_audio(self, src):
        """
        Play audio with possibility to kill the process
        :param src:
        :return: Subprocess playing audio which can be killed
        """
        cmd = 'exec mpg321 ' + src
        return subprocess.Popen(cmd, stdout=subprocess.PIPE, shell=True)

    def play_bytes_with_interruption(self, audio_bytes):
        """
        Transforms the bytes to mp3 file and start playing it
        :param audio_bytes:
        :return: True if the whole audio was played. False if we press interrupt button during listening the audio
        """
        self.logger.info("play_bytes")

        src = "./temp/" + self.DEVICE_ID + "_" + str(time.time()) + ".mp3"
        with open(src, "wb") as f:
            # Write the response to the output file.
            f.write(audio_bytes)

        p = self.play_audio(src)

        while p.poll() is None:
            if self.should_stop_audio:
                self.logger.debug("should_stop_audio==True")
                p.kill()
                os.remove(src)
                self.end_session()
                return False
            else:
                # self.logger.debug("sleeping")
                time.sleep(0.1)

        os.remove(src)
        return True

    def start_session(self):
        self.session_started = True
        self.should_stop_audio = False

    def end_session(self):
        self.session_started = False
        self.should_stop_audio = True

    def download_offline_sound(self):
        # Download the file if it does not exist
        if not os.path.isfile(no_internet):
            import urllib.request
            urllib.request.urlretrieve("https://promethist.ai/repository/misc/no_internet.mp3", no_internet)
        if not os.path.isfile(lost_internet):
            import urllib.request
            urllib.request.urlretrieve("https://promethist.ai/repository/misc/lost_internet.mp3", lost_internet)
        if not os.path.isfile(interspeech_sound):
            import urllib.request
            urllib.request.urlretrieve("https://promethist.ai/repository/misc/interspeech_sound.mp3", interspeech_sound)

    def run(self):
        self.download_offline_sound()

        self.button.when_pressed = self.button_pressed

        self.communication = WebsocketCommunication(host=self.HOST,
                                                    logger_name=LOGGER_NAME,
                                                    device_id=self.DEVICE_ID,
                                                    app_key="brainquist",
                                                    led_on=self.led.on,
                                                    led_off=self.led.off,
                                                    on_session_start=self.start_session,
                                                    on_session_end=self.end_session,
                                                    play_bytes=self.play_bytes_with_interruption,
                                                    play_audio=self.play_audio)
        while True:
            if "DEBUG" in os.environ:
                websocket.enableTrace(True)
            try:
                thread.start_new_thread(self.communication.run, ())
                while True:
                    print(str(self.communication.websocket.run_forever(ping_interval=60, ping_timeout=20)))
                    time.sleep(5)
                    self.logger.debug("Connection with port is lost or unavailable!")
            except ConnectionRefusedError as e:
                self.play_audio(no_internet)
                time.sleep(20)
            except (SystemExit, Exception):
                self.end_session()
                self.logger.error(traceback.format_exc())
                self.communication.websocket.close()
                time.sleep(4)
